# CocosBuilder - User Guide

With CocosBuilder you can graphically layout your Cocos2D node graphs and scenes and add them to your project with a single line of code. CocosBuilder is built around a plug-in system that allows the addition of almost any type of Cocos2D objects. By default, the following objects are supported _CCNode_, _CCLayer_, _CCLayerColor_, _CCLayerGradient_, _CCSprite_, _CCMenu_, _CCMenuItemImage_, _CCLabelBMFont_, _CCLabelTTF_, _CCParticleSystem_.


## Working in CocosBuilder

When creating a new document, make sure it is in the same directory as you other assets you want to use with CocosBuilder and in your project. Only assets in the same directory will show up in the program. The stage size can be up to a million points. The center of origin option will tell if the stage’s origin is set to the bottom left corner or the center of the stage. You can change the stage size and origin at any time in the View menu.

Use the selection tool to select objects to change their properties. Hold down the command key and drag the stage to instantly switch to the scroll view tool. Add objects by using the Object -> Add Object menu. You can also add CCSprite’s by dragging images from the Image Assets palette to the stage. If you drop images on a CCMenu they will automatically be added as CCMenuItemImages.

Edit the objects you have added by selecting them. If the object you have clicked on is obscured by another object, you can use the Select Behind option in the Object menu right after clicking. When an object is selected its properties will show up on the right side of the main window. Any updates you make in the properties view will be immediately reflected in your objects. You can move, rotate and scale objects by dragging their transform handles, it’s also possible to nudge them by using cmd-arrowkeys.


## Loading scenes and nodes

CocosBuilder documents, or ccb-files, needs to be published into a compact binary format, ccbi, before they can be loaded into your application. Once published they can be easily loaded with a single line of code. To load a node graph, add the CCBReader.h and CCBReader.m files to your Cocos2D project, then call the _nodeGraphFromFile:_ method as follows.

    CCNode* myNode = [CCBReader nodeGraphFromFile:@"MyNodeGraph.ccb"];

You may need to cast the returned value depending on what sort of object is the root node in your ccb-file and how you will use it in your code. For instance, if you load a CCParticleSystem, use the following code.

    CCParticleSystem* myParticles = (CCParticleSystem*) [CCBReader nodeGraphFromFile:@"MyParticleSystem.ccb"];

For your convenience, CCBReader can also wrap your node graph in a scene. To load your ccb-file in a scene call _sceneWithNodeGraphFromFile:_

    CCScene* myScene = [CCBReader sceneWithNodeGraphFromFile:@"MyScene.ccb"];


## Using custom classes

The way you can link your code with CocosBuilder is to use custom classes. To assign a custom class to an object in CocosBuilder, just select the object then enter the name of your custom class in the property inspector. Remember that your custom class needs to be a sub class of the selected object. When loading the ccbi-file CCBReader will initialize your custom class using the super class’s _init_ method, then set all of the objects properties. If you need to do any extra initialization of your object after the properties are set, CCBReader will call the _didLoadFromCCB_ method.


## Linking member variables

References to objects in your ccbi-file can be linked to member variables when the file is loaded. These member variables can be either in the root node of the document, in which case it must have been assigned a custom class. You can also assign them to a custom object, which is optionally passed as the _owner_ object to the CCBReader.

To load ccbi-file and assign members to it’s owner, the node graph needs to be loaded by calling either the _nodeGraphFromFile:owner:_ or the _sceneWithNodeGraphFromFile:owner:_ methods.

To link an object, simply declare them in the header file, either in the object you plan to pass as the owner or in the custom class of your document root object. Select the object in CocosBuilder, under Code Connections change the _Don’t assign_ popup menu to either _Doc_ root var or _Owner var_. Then enter the name of your member variable to the right of the popup menu.


## Adding callbacks to buttons

To add a callback when a CCMenuItemImage is tapped, simply select the CCMenuItemImage in CocosBuilder, then add the name of the method you want to call in the _Selector_ field. Set the target either to the _Document root_ or the _Owner_.


## Next steps

If you are familiar with Cocos2D, using CocosBuilder should be easy and straight forward. To get started, download the example to see how you can integrate CocosBuilder node graphs with your code.