# CocosBuilder - CCB File Format

This is a description of the CocosBuilder save file format. It is also used internally in CocosBuilder and is used by export plug-ins. Published files are (by default) saved in the CCBi file format.


## Structure

CCB files are simple Plist-files with a ccb extension. To dissect a CCB file in Xcode, simple rename the extension to .plist and open it with Xcode.


### Document (top level)

The root node in a CCB file is a dictionary. It contains the following properties.

<table>
    <tr>
        <th>Key</th><th>Type</th><th>Comment</th>
    </tr>
    <tr>
        <td>fileType</td><td>String</td><td>Must be CocosBuilder</td>
    </tr>
    <tr>
        <td>fileVersion</td><td>Number</td><td>Must be 3</td>
    </tr>
    <tr>
        <td>stageWidth</td><td>Number</td><td>The width of the stage in points</td>
    </tr>
    <tr>
        <td>stageHeight</td><td>Number</td><td>The height of the stage in points</td>
    </tr>
    <tr>
        <td>centeredOrigin</td><td>Boolean</td><td>Yes if the stage origin is centered, no if it is bottom left</td>
    </tr>
    <tr>
        <td>exportPlugIn</td><td>String</td><td>Optional, the extension of the export plug-in last used</td>
    </tr>
    <tr>
        <td>exportPath</td><td>String</td><td>Optional, the full path to the file last published</td>
    </tr>
    <tr>
        <td>nodeGraph</td><td>Dictionary</td><td>Node (as defined below)</td>
    </tr>
</table>


### Node

Represents a CCNode or a sub class of a CCNode.

<table>
    <tr>
        <th>Key</th><th>Type</th><th>Comment</th>
    </tr>
    <tr>
        <td>baseClass</td><td>String</td><td>Name of base class plug-in used, e.g. CCNode or CCSprite</td>
    </tr>
    <tr>
        <td>customClass</td><td>String</td><td>Name of super class defined in editor, e.g. HelloCocosBuilder</td>
    </tr>
    <tr>
        <td>memberVarAssignmentName</td><td>String</td><td>Name of member var this node is assigned to</td>
    </tr>
    <tr>
        <td>memberVarAssignmentType</td><td>Number</td><td>0 = don't assign, 1 = assign to root node, 2 = assign to owner</td>
    </tr>
    <tr>
        <td>properties</td><td>Array</td><td>Array of Property (as defined below), this defines all properties that are assigned to the node</td>
    </tr>
    <tr>
        <td>children</td><td>Array</td><td>Array of Node, list of all children's of this node</td>
    </tr>
</table>


### Property

Represents a property of a Node. Too see how different properties are serialized, please see _Property Types_.

<table>
    <tr>
        <th>Key</th><th>Type</th><th>Comment</th>
    </tr>
    <tr>
        <td>type</td><td>String</td><td>Name of the type of property, property types are described below</td>
    </tr>
    <tr>
        <td>name</td><td>String</td><td>Name of the property, e.g. contentSize or position</td>
    </tr>
    <tr>
        <td>value</td><td>n/a</td><td>Different property types are serialized in different ways</td>
    </tr>
    <tr>
        <td>platform</td><td>String</td><td>Optional, can be iOS or Mac. Set if a property is only supported for a specific platform</td>
    </tr>
</table>