# CocosBuilder - CCBi File Format

This is a description of the CocosBuilder export (publish) file format for Cocos2d-iphone. It is a binary file format designed to be as compact as possible and very quick to load.


## Basic types

The file format is built upon a set of basic types, such as integers, floats and strings.


### BYTE

A single unsigned integer saved in a byte


### BOOLEAN

Saved as a 0 or 1 in a single byte


### UINT

A unsigned integer, stored using Elias gamma encoding ( http://en.wikipedia.org/wiki/Elias_gamma_coding ). Before being stored, 1 is added to the number to support 0 values. The last byte is padded with 0:s so that the next written value is aligned to even bytes.


### SINT

A signed integer, stored using Elias gamma encoding. Before being stored, the number is transformed to a positive integer using bijection. 0 -> 1, -1 -> 2, 1 -> 3, -2 -> 4, 2 -> 5, etc.


### FLOAT

The first byte of the float defines how it is stored. Possible values are:

* 0 -> 0.0f
* 1 -> 1.0f
* 2 -> -1.0f
* 3 -> 0.5f
* 4 -> Saved as INT
* 5 -> Saved as full 4 byte float number


### STRING

Strings are saved as Java-style UTF8:s. The first two bytes define the length of the string, then the string is saved using UTF8 encoding (without a trailing \0);


## Strings and String cache


### STRING_CACHE

To save space, all strings are written to a string cache at the beginning of the file.

<table>
    <tr>
        <th>Type</th><th>Name</th><th>Description</th>
    </tr>
    <tr>
        <td>UINT</td><td>numStrings</td><td>Number of strings in the string cache</td>
    </tr>
    <tr>
        <td>STRING</td><td>str[0]</td><td>First string in cache</td>
    </tr>
    <tr>
        <td>STRING</td><td>str[1]</td><td>Second string in cache</td>
    </tr>
    <tr>
        <td>…</td><td></td><td></td>
    </tr>
    <tr>
        <td>STRING</td><td>str[numStrings-1]</td><td>Last string in cache</td>
    </tr>
</table>


### CSTRING

A cached string is saved using a single UINT which refers to the strings index in the string cache.


## Header


### HEADER

The header is used to ensure the file has the right type and version.

<table>
    <tr>
        <th>Type</th><th>Name</th><th>Description</th>
    </tr>
    <tr>
        <td>BYTE</td><td>magic0</td><td>Must be 'i'</td>
    </tr>
    <tr>
        <td>BYTE</td><td>magic1</td><td>Must be 'b'</td>
    </tr>
    <tr>
        <td>BYTE</td><td>magic2</td><td>Must be 'c'</td>
    </tr>
    <tr>
        <td>BYTE</td><td>magic3</td><td>Must be 'c'</td>
    </tr>
    <tr>
        <td>UINT</td><td>version</td><td>Must be 1</td>
    </tr>
</table>


## Node graph


### PROPERTY

Represents a property of a node. For type IDs and how they are serialized, see the _Property Types_ document.

<table>
    <tr>
        <th>Type</th><th>Name</th><th>Description</th>
    </tr>
    <tr>
        <td>UINT</td><td>typeID</td><td>ID of the type</td>
    </tr>
    <tr>
        <td>CSTRING</td><td>propertyName</td><td>The name of the property</td>
    </tr>
    <tr>
        <td>BYTE</td><td>platform</td><td>The type of platform the property is supported for. Can be 0 = any platform, 1 = iOS only, 2 = Mac only</td>
    </tr>
    <tr>
        <td>n/a</td><td>serializedValue</td><td>Value serialized as described in the Property Types document</td>
    </tr>
</table>


### NODE

Represents a node or a node graph (if the node has children).

<table>
    <tr>
        <th>Type</th><th>Name</th><th>Description</th>
    </tr>
    <tr>
        <td>CSTRING</td><td>class</td><td>Name of the nodes class</td>
    </tr>
    <tr>
        <td>UINT</td><td>memberVarAssignmentType</td><td>The target this node should be assigned as a variable for. 0 = No target, 1 = documents root node, 2 = owner (as passed to CCBReader)</td>
    </tr>
    <tr>
        <td>CSTRING</td><td>memberVarAssignmentName</td><td>The name of the variable this node should be assigned to (only written if memberVarAssignmentType != 0)</td>
    </tr>
    <tr>
        <td>UINT</td><td>numProperties</td><td>Number of properties saved for this node</td>
    </tr>
    <tr>
        <td>PROPERTY</td><td>property[0]</td><td>The first property of this node</td>
    </tr>
    <tr>
        <td>PROPERTY</td><td>property[1]</td><td>The second property of this node</td>
    </tr>
    <tr>
        <td>…</td><td></td><td></td>
    </tr>
    <tr>
        <td>PROPERTY</td><td>property[numProperties-1]</td><td>The last property of this node</td>
    </tr>
    <tr>
        <td>UINT</td><td>numChildren</td><td>Number of children of this node</td>
    </tr>
    <tr>
        <td>NODE</td><td>child[0]</td><td>The first child of this node</td>
    </tr>
    <tr>
        <td>NODE</td><td>child[1]</td><td>The second child of this node</td>
    </tr>
    <tr>
        <td>…</td><td></td><td></td>
    </tr>
    <tr>
        <td>NODE</td><td>child[numProperties-1]</td><td>The last child of this node</td>
    </tr>
</table>


## Overall document structure


### DOCUMENT

The top structure of a CCBi document.

<table>
    <tr>
        <th>Type</th><th>Name</th><th>Description</th>
    </tr>
    <tr>
        <td>HEADER</td><td>header</td><td>Document header</td>
    </tr>
    <tr>
        <td>STRING_CACHE</td><td>stringCache</td><td>All strings referenced by this file</td>
    </tr>
    <tr>
        <td>NODE</td><td>rootNode</td><td>The root node of this document (which can also have children)</td>
    </tr>
</table>